;; glog_sfq.el: FAQ(SGMLform) mode functions for GLOG. -*-Emacs-Lisp-*-
;; Copyright (C) 1994-95 Namiki Seiko (Nam)
;; (E-mail: CAF58901@pcvan.or.jp, RGE01545@niftyserve.or.jp)
;;
;; Everyone is granted permission to copy, modify and redistribute,
;; but WITHOUT ANY WARRANTY.
;;
(defconst glog_sfq "$Header: glog_sfq.elv  1.5  95/08/05 22:59:30  Nam  Exp $")

(require 'glog)

(defun view-sgmlfaq-mode (&optional view-return-to-buffer)
  "Major mode for viewing SGML form FAQ."
  (fset 'glog-get-forum-name      (function glog-get-chapter-sgmlfaq))
  (fset 'glog-get-board-name      (function glog-get-section-sgmlfaq))
  (fset 'glog-get-article-info    (function glog-get-subject-sgmlfaq))
  (fset 'glog-get-mail-info       (function glog-undefined-function))
  (fset 'glog-set-post-header     (function glog-undefined-function))
  (fset 'glog-set-post-terminator (function glog-undefined-function))
  (fset 'glog-set-mail-header     (function glog-undefined-function))
  (fset 'glog-set-mail-terminator (function glog-undefined-function))
  (fset 'glog-check-post          (function glog-undefined-function))
  (setq glog-match-string-forum     "^\<chap\>")
  (setq glog-match-string-board     "^\<sect\>")
  (setq glog-match-string-article   "^\<FAQ\>$")
  (setq glog-match-string-mail      nil)
  (setq glog-match-string-skip      nil)
  (setq glog-match-string-article-end
        "^\</FAQ\>$")
  (setq glog-search-regexp
        (concat
	 "\\(" glog-match-string-forum   "\\|" glog-match-string-board
	 "\\|" glog-match-string-article "\\)"))
  (setq glog-sgmlfaq-subject-count 0)
  (run-hooks 'view-sgmlfaq-mode-hook))

(defun glog-sgmlfaq-version ()
  "elispversion(RCSwb_)Ԃ."
  glog_sfq)

(defun glog-get-chapter-sgmlfaq ()
  "SGML`chapter𓾂."
  (let ((linebuf (glog-get-current-line))(chap nil)(foo)(bar))
    (if (string-match "\<ct\>" linebuf)
	(progn
	  (setq foo (match-end 0))
	  (if (setq bar (string-match "\</ct\>" linebuf))
	      (setq chap (substring linebuf foo bar))
	    (setq chap (substring linebuf foo))))
      (setq chap linebuf))
    chap))

(defun glog-get-section-sgmlfaq ()
  "SGML`section𓾂."
  (let ((linebuf (glog-get-current-line))(sect nil)(foo)(bar))
    (if (string-match "\<st\>" linebuf)
	(progn
	  (setq foo (match-end 0))
	  (if (setq bar (string-match "\</st\>" linebuf))
	      (setq sect (substring linebuf foo bar))
	    (setq sect (substring linebuf foo))))
      (setq sect linebuf))
    sect))

(defun glog-search-tag-sgmlfaq (tag &optional def limit)
  "TAGT"
  (save-excursion
    (let ((linebuf)(foo)(bar)(res))
      (setq res (if def def ""))
      (if limit () (setq limit (glog-get-forward-line-position 10)))
      (if (re-search-forward tag limit 1)
	  ;; TAG
	  (progn
	    (setq linebuf (glog-get-current-line))
	    (if (string-match tag linebuf)
		(setq res (substring linebuf (match-end 0))))))
      res)))

(defun glog-get-subject-sgmlfaq ()
  "SGML`subject̏𓾂."
  (let ((date "????-??-?? ??:??:??")(author "")(title "**")
	(pos)(lines)(hlines 1)(art-no -1)(com-no)(linebuf)(limit)(foo)(bar))
    ;; Jnʒu
    (forward-line)
    (setq pos (point))
    (beginning-of-line)
    (setq limit (glog-get-forward-line-position 19))
    ;; {[h擾
    (setq title (glog-search-tag-sgmlfaq "^\<Subject\>" title limit))
    ;; Җ擾
    (setq author (glog-search-tag-sgmlfaq "^\<Poster\>" author limit))
    ;; 擾
    (setq date (glog-search-tag-sgmlfaq "^\<Date\>" date limit))
    ;; ԍ͏d͖̂ƂĘAԂ
    (setq art-no
	  (setq glog-sgmlfaq-subject-count (+ glog-sgmlfaq-subject-count 1)))
    ;; ŏ<Q><A>܂łwb_s
    (save-excursion
      (if (re-search-forward "^\<[QA]\>" limit 1)
	  (setq hlines (- (count-lines pos (point)) 1))))
    ;; s
    (setq limit (glog-get-forward-line-position 300)) ; get search end pos
    (re-search-forward glog-match-string-article-end limit 1)
    (beginning-of-line)
    (setq lines (count-lines pos (point)))
    (if (< lines 2)
	nil
      ;; XgɓZ߂
      ;; (ԍ unread title id nh  Rg ʒu s
      ;;  wb_s )
      (list art-no ?  title author author date com-no pos lines
	    hlines 'board))))

(provide 'glog_sfq)

;;; glog_sfq.el ends here
